package com.huimv.environ.env.timer;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSONObject;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huimv.environ.eco.entity.envVo.EnvVo;
import com.huimv.environ.eco.service.IBaseTelecomRoomService;
import com.huimv.environ.env.environ.dao.entity.EnvDayEntity;
import com.huimv.environ.env.environ.dao.repo.EnvDayRepo;
import com.mysql.cj.xdevapi.JsonArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.cache.CacheProperties;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.*;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Component
@EnableScheduling
public class EnvTimer {
    @Autowired
    private RedisTemplate<String ,String> redisTemplate;

    @Autowired
    private EnvDayRepo envDayRepo;

    @Scheduled(cron = "0 6 * * * ? ")
    private void get() throws JsonProcessingException {
        RestTemplate restTemplate = new RestTemplate();
        String envUrl = "https://dnqyt.envchina.com/api/public/station-data";
        LocalDate previousDay = LocalDate.now().minusDays(1);
        LocalDateTime previousDayStart = LocalDateTime.of(previousDay, LocalTime.MIN);
        String formattedDateTime = previousDayStart.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Date date = Date.from(previousDayStart.atZone(ZoneId.systemDefault()).toInstant());
        String sgToken = redisTemplate.opsForValue().get("sg:token");
        System.out.println(sgToken);
        //token不存在重新获取
        if (StringUtils.isEmpty(sgToken)){
            String url = "https://dnqyt.envchina.com/api/public/token";
            HashMap<String, String> params = new HashMap<>();
            params.put("userName", "ZXQLYZKJ2307A2");
            params.put("password", "e91294970cb74aaf9d9a6fe0973d8f5e");
            // 设置请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);
            // 创建请求实体对象
            HttpEntity<HashMap<String, String>> requestEntity = new HttpEntity<>(params, headers);
            // 发送POST请求
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(url, requestEntity, String.class);
            JSONObject jsonObject = JSONObject.parseObject(responseEntity.getBody());
            if (jsonObject.getInteger("code") != 200){
                System.out.println("获取token失败!");
            }else {
                sgToken = jsonObject.getJSONObject("data").getString("token");
                redisTemplate.opsForValue().set("sg:token", sgToken,2L,TimeUnit.HOURS);
            }
        }
        HttpHeaders headers = new HttpHeaders();
        headers.add("token",sgToken);
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<Map<String, Object>> formEntity = new HttpEntity<Map<String, Object>>(headers);

        String urlWithParams = envUrl + "?type=D&mn=33330424001391" + "&startTime=" + formattedDateTime + "&endTime=" + formattedDateTime;
        System.out.println(formattedDateTime);
        ResponseEntity<String> result = restTemplate.exchange(urlWithParams, HttpMethod.GET,formEntity,String.class);
        System.out.println(result);
        JSONObject jsonObject = JSONObject.parseObject(result.getBody());
        if (jsonObject.getInteger("code") == 200){
            EnvDayEntity envDayEntity = new EnvDayEntity();
            String data = jsonObject.getString("data");
            ObjectMapper objectMapper = new ObjectMapper();
            List<EnvVo> dataList = objectMapper.readValue(data, new TypeReference<List<EnvVo>>() {});
            for (EnvVo envVo : dataList) {
                if (envVo.getFactorCode().equals("w00000")){
                    envDayEntity.setWs(envVo.getAvg());
                }
                if (envVo.getFactorCode().equals("w01001")){
                    envDayEntity.setPh(envVo.getAvg());
                }
                if (envVo.getFactorCode().equals("w01018")){
                    envDayEntity.setCod(envVo.getAvg());
                }
                if (envVo.getFactorCode().equals("w21003")){
                    envDayEntity.setAd(envVo.getAvg());
                }
            }
            envDayEntity.setAddDate(date);
            envDayEntity.setFarmId(1);
            envDayRepo.save(envDayEntity);
        }

    }

    public static void main(String[] args) throws JsonProcessingException {

    }

}
